# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'explorerwidgetui.ui'
#
# Created: Mon Sep 24 12:30:21 2012
#      by: PyQt4 UI code generator 4.9.1
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

class Ui_RprjExplorerWidget(object):
    def setupUi(self, RprjExplorerWidget):
        RprjExplorerWidget.setObjectName(_fromUtf8("RprjExplorerWidget"))
        RprjExplorerWidget.resize(359, 483)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(_fromUtf8(":/rPrj/icons/rprj.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        RprjExplorerWidget.setWindowIcon(icon)
        self.verticalLayout = QtGui.QVBoxLayout(RprjExplorerWidget)
        self.verticalLayout.setMargin(0)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.widget = QtGui.QWidget(RprjExplorerWidget)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.widget.sizePolicy().hasHeightForWidth())
        self.widget.setSizePolicy(sizePolicy)
        self.widget.setObjectName(_fromUtf8("widget"))
        self.horizontalLayout = QtGui.QHBoxLayout(self.widget)
        self.horizontalLayout.setMargin(0)
        self.horizontalLayout.setMargin(0)
        self.horizontalLayout.setObjectName(_fromUtf8("horizontalLayout"))
        self.searchFolder = QtGui.QLineEdit(self.widget)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.searchFolder.sizePolicy().hasHeightForWidth())
        self.searchFolder.setSizePolicy(sizePolicy)
        self.searchFolder.setObjectName(_fromUtf8("searchFolder"))
        self.horizontalLayout.addWidget(self.searchFolder)
        self.comboForms = QtGui.QComboBox(self.widget)
        self.comboForms.setSizeAdjustPolicy(QtGui.QComboBox.AdjustToContents)
        self.comboForms.setObjectName(_fromUtf8("comboForms"))
        self.horizontalLayout.addWidget(self.comboForms)
        self.searchButton = QtGui.QPushButton(self.widget)
        self.searchButton.setText(_fromUtf8(""))
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(_fromUtf8(":/rPrj/icons/Zoom16.gif")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.searchButton.setIcon(icon1)
        self.searchButton.setObjectName(_fromUtf8("searchButton"))
        self.horizontalLayout.addWidget(self.searchButton)
        self.horizontalLayout.setStretch(0, 1)
        self.verticalLayout.addWidget(self.widget)
        self.treeView = QtGui.QTreeView(RprjExplorerWidget)
        self.treeView.setEditTriggers(QtGui.QAbstractItemView.NoEditTriggers)
        self.treeView.setDragEnabled(True)
        self.treeView.setDragDropMode(QtGui.QAbstractItemView.DragDrop)
        self.treeView.setDefaultDropAction(QtCore.Qt.MoveAction)
        self.treeView.setSortingEnabled(True)
        self.treeView.setAnimated(True)
        self.treeView.setObjectName(_fromUtf8("treeView"))
        self.verticalLayout.addWidget(self.treeView)

        self.retranslateUi(RprjExplorerWidget)
        QtCore.QMetaObject.connectSlotsByName(RprjExplorerWidget)

    def retranslateUi(self, RprjExplorerWidget):
        RprjExplorerWidget.setWindowTitle(QtGui.QApplication.translate("RprjExplorerWidget", "Explorer", None, QtGui.QApplication.UnicodeUTF8))
        self.searchFolder.setToolTip(QtGui.QApplication.translate("RprjExplorerWidget", "Search items", None, QtGui.QApplication.UnicodeUTF8))
        self.searchButton.setToolTip(QtGui.QApplication.translate("RprjExplorerWidget", "Search", None, QtGui.QApplication.UnicodeUTF8))
        self.searchButton.setShortcut(QtGui.QApplication.translate("RprjExplorerWidget", "Ctrl+F", None, QtGui.QApplication.UnicodeUTF8))

import apps_rc



# -*- coding: utf-8 -*-

#
# @copyright &copy; 2012 by Roberto Rocco Angeloni <roberto@roccoangeloni.it>
# @license http://opensource.org/licenses/lgpl-3.0.html GNU Lesser General Public License, version 3.0 (LGPLv3)
# @version $Id: editwidgetcontroller.py $
# @package qtwidgets
# 
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

"""

pyuic4 explorerwidgetui.ui -o explorerwidgetui.py

Signals:
updatedTree()
selected(QStandardItem,PyQt_PyObject)

"""
import os,sys,traceback
from PyQt4 import QtCore,QtGui

try:
	_fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
	_fromUtf8 = lambda s: s

import explorerwidgetui
from rprj.formulator import FAssociation

DEBUG_WIDGET=True

class ExplorerWidget(QtGui.QWidget):
	def __init__(self,parent,objectName):
		QtGui.QWidget.__init__(self,parent)
		# UI
		self.ui = explorerwidgetui.Ui_RprjExplorerWidget()
		self.ui.setupUi(self)
		self.setObjectName(objectName)
		# Tree Model
		self.modelTree = QtGui.QStandardItemModel( 0, 1 )
		self.modelTree.setHorizontalHeaderItem( 0, QtGui.QStandardItem("Name") )
		self.ui.treeView.setModel( self.modelTree )
		self.ui.treeView.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
		# Slots
		self.bind()
		# Logic
		self.server = None
		self.formFactory = None
		self._searchTypes = [] #'DBEObject']
		# Container
		self.container = None
		# DragDrop
		self.setAcceptDrops(True)
	def bind(self):
		# Search
		QtCore.QObject.connect(self.ui.searchFolder,QtCore.SIGNAL("returnPressed()"),self.slotSearchFolder)
		QtCore.QObject.connect(self.ui.searchButton,QtCore.SIGNAL("clicked()"),self.slotSearchFolder)
		# Tree View
		QtCore.QObject.connect(self.ui.treeView,QtCore.SIGNAL("clicked(QModelIndex)"),self.slotTreeViewClick)
		QtCore.QObject.connect(self.ui.treeView,QtCore.SIGNAL("doubleClicked(QModelIndex)"),self.slotTreeViewDoubleClick)
		QtCore.QObject.connect(self.ui.treeView,QtCore.SIGNAL("customContextMenuRequested(QPoint)"),self.slotTreeViewContextMenu)
		QtCore.QObject.connect(self.ui.treeView,QtCore.SIGNAL("activated(QModelIndex)"),self.slotTreeViewClick)
		# Model Tree
		QtCore.QObject.connect(self.modelTree,QtCore.SIGNAL("itemChanged(QStandardItem*)"),self.slotItemChanged)

# ############################## DRAG DROP ###################################
	def dragEnterEvent(self,event):
		"""called when a drag is in progress and the mouse enters the DropArea object"""
		childOver = self.childAt(event.pos())
		if self.ui.treeView.viewport() == self.childAt(event.pos()):
			event.acceptProposedAction()
	def dragMoveEvent(self,event):
		childOver = self.childAt(event.pos())
		if self.ui.treeView.viewport() == childOver:
			item = self._treeItemAtEventPosition(event.pos())
			if item is None:
				return
			self.runSearchChilds( item )
			mydata = item.data().toPyObject()
			event.acceptProposedAction()
	def _treeItemAtEventPosition(self,eventPosition):
		item = None
		childOver = self.childAt(eventPosition)
		if self.ui.treeView.viewport() == childOver:
			headersize=QtCore.QPoint(0,self.ui.treeView.header().size().height())
			deltapos = eventPosition - self.ui.treeView.pos() - headersize
			index = self.ui.treeView.indexAt(deltapos)
			item = self.modelTree.itemFromIndex(index)
			if not item is None and item.hasChildren() and not self.ui.treeView.isExpanded(item.index()):
				self.ui.treeView.setExpanded(item.index(),True)
			else:
				if DEBUG_WIDGET: print "ExplorerWidget._treeItemAtEventPosition: eventPosition=%s"%(eventPosition)
		return item
	def dragLeaveEvent(self,event):
		"""called when a drag is in progress and the mouse leaves the widget"""
		event.accept()
	def dropEvent(self,event):
		myformat = event.format()
		item = self._treeItemAtEventPosition(event.pos())
		if item is None:
			return
		mydata = item.data().toPyObject()
		_mytypename = mydata.getTypeName()
		if _mytypename=='DBEObject':
			_mytypename=mydata.getValue('classname')
		myform = self.formFactory.getInstanceByDBEName( _mytypename, dbmgr=self.server)
		if myform.getDetailFormsCount()==0:
			return
		myformlist = [myform.getDetail(x) for x in range(myform.getDetailFormsCount())]
		mimeData = event.mimeData()
		if myformat=='text/uri-list' or myformat=='x-special/gnome-icon-list':
			for u in mimeData.urls():
				myurl = "%s"%u.toString()
				if not myurl.startswith("file://"):
					continue
				myfilename = myurl.replace("file://","")
				if not os.path.exists(myfilename):
					continue
				import mimetypes
				mime,altro = mimetypes.guess_type( myurl )
				newdbe = self.server.getClazzByTypeName("DBEFile")(attrs={'filename':myfilename})
				if mime=='message/rfc822':
					newdbe = self.server.getClazzByTypeName("DBEMail")(attrs={'filename':myfilename})
				myform = self.formFactory.getInstanceByDBEName(newdbe.getTypeName(),dbmgr=self.server)
				newdbe.readFKFrom( mydata )
				newdbe.setDefaultValues(self.server)
				newdbe.setValue('father_id',mydata.getValue('id'))
				newdbe.setValue('name',"New %s"%newdbe.getTypeName())
				newdbe = self.server.insert(newdbe)
				icon = QtGui.QIcon()
				icon.addPixmap(QtGui.QPixmap(_fromUtf8(":/rPrj/%s"%myform.getDetailIcon())), QtGui.QIcon.Normal, QtGui.QIcon.Off)
				child = QtGui.QStandardItem( icon, "%s" % (newdbe.getValue('name') ) )
				child.setData( newdbe )
				item.appendRow( child )
		else:
			self.SetStatusText("DND: format '%s' not allowed." % myformat)
# ############################## PROPERTIES ###################################
	def setServer(self,server):
		self.server = server
		self.modelTree.server = server
	def setFormFactory(self,formFactory):
		self.formFactory = formFactory
		# Update UI
		pass
		# Cleaning comboForms
		for i in range(self.ui.comboForms.count()):
			self.ui.comboForms.removeItem(i)
		self.ui.comboForms.addItem("","DBEObject")
		formList = self.formFactory.getAllClassnames()
		for _f in formList:
			if _f in ['default','FBanned','FLog','FObject'
						,'FProjectPeopleRole','FProjectCompanyRole','FTodoTipo','FUserGroupAssociation']:
				continue
			if _f.find("Filter")>=0:
				continue
			myform = self.formFactory.getInstance(_f, dbmgr=self.server)
			if isinstance(myform,FAssociation):
				continue
			# comboForms
			icon = QtGui.QIcon()
			icon.addPixmap(QtGui.QPixmap(_fromUtf8(":/rPrj/%s"%myform.getDetailIcon())), QtGui.QIcon.Normal, QtGui.QIcon.Off)
			dbename = myform.getDBE(self.server).getTypeName()
			self.ui.comboForms.addItem(icon,myform.getListTitle(),dbename)
	def getName(self):
		d = self.server.getConnectionProvider().getDBType()
		u = "nobody"
		if not self.server.getDBEUser() is None:
			u = self.server.getDBEUser().getValue('login')
		h = "%s" % self.server.getConnectionProvider().getHost()
		if d=="SQLite":
			h = ( "%s" % self.server.getConnectionProvider().getDB() ).split( os.path.sep )[-1]
		if h.find("://")>0:
			_url = h.split("://")
			if len(_url)>1:
				h = "%s:%s" % ( _url[0],_url[1].split("/")[0] )
		return "%s@%s" % (u,h)
	def setSearchVisible(self, b):
		self.ui.widget.setVisible(b)
	def setSearchTypes(self,types):
		"""['DBEObject', 'DBEFile']"""
		self._searchTypes = types
	def getCurrentDBE(self):
		"""Returns the current selected DBE in the tree"""
		index = self.ui.treeView.currentIndex()
		#print "ExplorerWidget.getCurrentDBE: index=%s"%index
		myitem = self.modelTree.itemFromIndex(index)
		if myitem is None:
			return None
		mydbe = myitem.data().toPyObject()
		return mydbe
		#if mydbe.isNew():
		#	return
# ############################## PREFERENCES ###################################
	def getPref(self,section,option,default=None):
		if self.container is None:
			return default
		return self.container.getPref(section,option,default)
	def setPref(self,section,option,valore):
		if self.container is None:
			return
		self.container.setPref(section,option,valore)
# ############################## ... ###################################
	def Alert(self,msg, eccezione=None):
		if self.container is None:
			return
		self.container.Alert(msg,eccezione)
	def Confirm(self,msg):
		if self.container is None:
			return True
		return self.container.Confirm(msg)
	def SetStatusText(self,msg):
		if self.container is None:
			return
		self.container.SetStatusText(msg)
	def SetProgress(self,perc):
		if self.container is None:
			return
		self.container.SetProgress(perc)
# ############################## UI ###################################
	def removeItem(self,myitem):
		myparent = myitem.parent()
		myrow = myitem.index().row()
		myparent.removeRow(myrow)
		self.ui.treeView.setCurrentIndex( self.modelTree.indexFromItem(myparent) )
	def updateTree(self,lista):
		self.modelTree.clear()
		# 2012.11.11: start.
		self.ids_items={}
		self.fatherids_items={}
		rootItem = self.modelTree.invisibleRootItem()
		self.updateItem(rootItem, lista)
		#for l in lista:
			#_mytypename = l.getTypeName()
			#if _mytypename=='DBEObject':
				#_mytypename=l.getValue('classname')
			#myform = self.formFactory.getInstanceByDBEName( _mytypename, dbmgr=self.server)
			#icon = QtGui.QIcon()
			#icon.addPixmap(QtGui.QPixmap(_fromUtf8(":/rPrj/%s"%myform.getDetailIcon())), QtGui.QIcon.Normal, QtGui.QIcon.Off)
			#item = QtGui.QStandardItem( icon, "%s" % (l.getValue('name')) )
			#item.setData(l)
			#desc = l.getValue('description')
			#if desc>'':
				#item.setToolTip( desc )
			#self.modelTree.appendRow( item )
		# 2012.11.11: end.
		self.modelTree.setHorizontalHeaderItem( 0, QtGui.QStandardItem("Name") )
		# TODO connect this signal to the first 8 lines of code in rPrj.updateTree
		self.emit(QtCore.SIGNAL("updatedTree()"))
	def updateItem(self,item,lista):
		rootItem = self.modelTree.invisibleRootItem()
		for l in lista:
			_mytypename = l.getTypeName()
			if _mytypename=='DBEObject':
				_mytypename=l.getValue('classname')
			myform = self.formFactory.getInstanceByDBEName( _mytypename, dbmgr=self.server)
			icon = QtGui.QIcon()
			icon.addPixmap(QtGui.QPixmap(_fromUtf8(":/rPrj/%s"%myform.getDetailIcon())), QtGui.QIcon.Normal, QtGui.QIcon.Off)
			child = QtGui.QStandardItem( icon, "%s" % (l.getValue('name') ) )
			child.setData(l)
			desc = l.getValue('description')
			if desc>'':
				child.setToolTip( desc )
			# 2012.11.11: start.
			self.ids_items[l.getValue('id')]=child
			if l.getValue('father_id')>'':
				if not self.fatherids_items.has_key(l.getValue('father_id')):
					self.fatherids_items[l.getValue('father_id')]=[]
				self.fatherids_items[l.getValue('father_id')].append(child)
			# 1. Check whether I'm son of somebody
			father_id = l.getValue('father_id')
			parentItem = None
			if father_id>'' and self.ids_items.has_key(father_id):
				parentItem = self.ids_items[father_id]
			#if father_id>'':
				#parentItem = self._findItemById(father_id)
			if not parentItem is None and not parentItem==rootItem and not parentItem==item:
				parentItem.appendRow( child )
				self.ui.treeView.setExpanded(parentItem.index(),True)
			else:
				item.appendRow( child )
			# 2. Check whether my childs are already here
			if self.fatherids_items.has_key(l.getValue('father_id')):
				mychilds = self.fatherids_items[l.getValue('father_id')]
				if len(mychilds)>0: print "TODO: che fare con i %s childs?"%len(mychilds)
			#item.appendRow( child )
			# 2012.11.11: end.
		self.ui.treeView.setExpanded(item.index(),True)
	def _findItemById(self,uuid,rootItem=None):
		if rootItem is None:
			rootItem = self.modelTree.invisibleRootItem()
		ret = self.modelTree.invisibleRootItem()
		for i in range(rootItem.rowCount()):
			_item = rootItem.child( i )
			try:
				_dbe = _item.data().toPyObject()
				if _dbe.getValue('id')==uuid:
					ret = _item
					break
				else:
					ret = self._findItemById(uuid,_item)
			except Exception,e:
				if DEBUG_WIDGET: print "_findItemById: ECCEZIONE=%s" % e
		return ret
# ############################## LOGIC ###################################
	def runSearchFolder(self,name,mydbetype='DBEObject'):
		if name != "%s" % self.ui.searchFolder.text():
			self.ui.searchFolder.setText(name)
		if mydbetype != "%s" % self.ui.comboForms.itemData( self.ui.comboForms.currentIndex() ).toPyObject():
			self.ui.comboForms.setCurrentIndex( self.ui.comboForms.findData(mydbetype) )
		attrname = 'name'
		if mydbetype=='DBEUser':
			attrname='login'
		cerca = self.server.getClazzByTypeName(mydbetype)(attrs={attrname:"%s" % name})
		lista = self.server.search(cerca, full_object=False)
		if len(lista)>0:
			if len(name)==0:
				tmp = []
				for dbe in lista:
					if mydbetype!="DBEObject" or dbe.getValue('father_id') is None or dbe.getValue('father_id')==0:
						tmp.append(dbe)
				lista = tmp
		self.updateTree(lista)
	def runSearch(self,name,mydbetype='DBEObject'):
		"""A better name..."""
		self.runSearchFolder(name,mydbetype)
	def runSearchChilds(self, item, refresh=False):
		mydata = item.data().toPyObject()
		if mydata.getValue('id') is None:
			return
		if refresh:
			item.removeRows( 0, item.rowCount() )
		if item.hasChildren():
			return
		# 2012.07.15: inizio.
		lista = []
		if len(self._searchTypes)>0:
			for dbename in self._searchTypes:
				cerca = self.server.getClazzByTypeName(dbename)(attrs={'father_id':mydata.id})
				lista.extend( self.server.search(cerca, full_object=False) )
		else:
			cerca = self.server.getClazzByTypeName('DBEObject')(attrs={'father_id':mydata.id})
			lista = self.server.search(cerca, full_object=False)
		#cerca = self.server.getClazzByTypeName('DBEObject')(attrs={'father_id':mydata.id})
		#lista = self.server.search(cerca, full_object=False)
		# 2012.07.15: fine.
		if len(lista)==0:
			return
		self.updateItem(item,lista)
	def runDelete(self,mydbe):
		"""returns: None=>OK, mydbe=>errors"""
		if mydbe.isNew():
			mydbe = None
		else:
			mydbe = self.server.delete(mydbe)
		return mydbe
# ############################## SLOTS ###################################
	def slotSearchFolder(self):
		name = "%s" % self.ui.searchFolder.text()
		mydbetype = "%s" % self.ui.comboForms.itemData( self.ui.comboForms.currentIndex() ).toPyObject()
		self.runSearchFolder(name,mydbetype)
	def slotTreeViewClick(self, index):
		item = self.modelTree.itemFromIndex(index)
		mydata = item.data().toPyObject()
		self.runSearchChilds( item )
		self.emit(QtCore.SIGNAL("selected(QStandardItem,PyQt_PyObject)"),item,mydata)
	def slotTreeViewDoubleClick(self, index):
		pass
	def slotTreeViewContextMenu(self,pos):
		index = self.ui.treeView.currentIndex()
		myitem = self.modelTree.itemFromIndex(index)
		# Defaults
		mydbe = self.server.getClazzByTypeName("DBEFolder")()
		myform = self.formFactory.getInstance("FFolder",dbmgr=self.server)
		if not myitem is None:
			mydbe = myitem.data().toPyObject()
			if mydbe.getTypeName()=='DBEObject':
				myform = self.formFactory.getInstanceByDBEName(mydbe.getValue('classname'),dbmgr=self.server)
			else:
				myform = self.formFactory.getInstanceByDBEName(mydbe.getTypeName(),dbmgr=self.server)
		else:
			myitem = self.modelTree.invisibleRootItem()
		menu = QtGui.QMenu(self.ui.treeView)
		icon = QtGui.QIcon()
		icon.addPixmap(QtGui.QPixmap(_fromUtf8(":/rPrj/icons/reload.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
		myaction = menu.addAction(icon,QtGui.QApplication.translate("TreeView", "Reload", None, QtGui.QApplication.UnicodeUTF8),self.slotReload)
		myaction.setData(None)
		# Build myformlist
		myformlist = []
		if myitem == self.modelTree.invisibleRootItem():
			formList = self.formFactory.getAllClassnames()
			for _f in formList:
				if _f in ['default','FBanned','FEventFilter','FLog','FObject'
							,'FProjectPeopleRole','FProjectCompanyRole','FTodoTipo','FUserGroupAssociation']:
					continue
				if _f.find("Filter")>=0:
					continue
				_myform = self.formFactory.getInstance(_f, dbmgr=self.server)
				if isinstance(_myform,FAssociation):
					continue
				myformlist.append(_myform)
		else:
			myformlist = [myform.getDetail(x) for x in range(myform.getDetailFormsCount())]
		myformlist = [x for x in myformlist if not isinstance(x,FAssociation)]
		if len(self._searchTypes)>0:
			tmpformlist = filter( lambda x: x.getDBE(self.server).getTypeName() in self._searchTypes, myformlist )
			myformlist = tmpformlist
		# Current form actions
		myactions = myform.getActions()
		if myactions.has_key('reload'): myactions.pop('reload')
		# Adding 'add child' actions
		if len(myformlist)>0 or len(myactions)>0:
			menu.addSeparator()
		if len(myformlist)>0:
			menuAdd = menu.addMenu(QtGui.QApplication.translate("TreeView", "Add"))
			for mydetail in myformlist:
				icon = QtGui.QIcon()
				icon.addPixmap(QtGui.QPixmap(_fromUtf8(":/rPrj/%s"%mydetail.getDetailIcon())), QtGui.QIcon.Normal, QtGui.QIcon.Off)
				myaction = menuAdd.addAction(icon,QtGui.QApplication.translate("TreeView", "%s"%mydetail.getDetailTitle(), None, QtGui.QApplication.UnicodeUTF8))#,self.slotTest)
				myaction.setData( mydetail.getDBE(self.server) )
		# Adding custom form actions
		if len(myactions.keys())>0:
			menuActions = menu.addMenu(QtGui.QApplication.translate("TreeView", "Actions"))
			for k,v in myactions.items():
				if len(myactions[k])>2:
					icon = QtGui.QIcon()
					# 2012.10.16: start.
					icon.addPixmap(QtGui.QPixmap(_fromUtf8(":/rPrj/icons/%s"%myactions[k][2])), QtGui.QIcon.Normal, QtGui.QIcon.Off)
					#icon.addPixmap(QtGui.QPixmap(_fromUtf8(":/rPrj/%s"%myactions[k][2])), QtGui.QIcon.Normal, QtGui.QIcon.Off)
					# 2012.10.16: end.
					myaction = menuActions.addAction(icon,QtGui.QApplication.translate("TreeView", myactions[k][0], None, QtGui.QApplication.UnicodeUTF8),self.slotCustomAction)
				else:
					myaction = menuActions.addAction(QtGui.QApplication.translate("TreeView", myactions[k][0], None, QtGui.QApplication.UnicodeUTF8),self.slotCustomAction)
				myaction.setData( "%s"%k )
		menu.addSeparator()
		icon = QtGui.QIcon()
		icon.addPixmap(QtGui.QPixmap(_fromUtf8(":/rPrj/icons/editdelete.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
		myaction = menu.addAction(icon,QtGui.QApplication.translate("TreeView", "Delete", None, QtGui.QApplication.UnicodeUTF8),self.slotDelete)
		myaction.setData(None)
		selectedAction = menu.exec_( QtGui.QCursor.pos() )
		if selectedAction is None:
			return
		newdbe = selectedAction.data().toPyObject()
		if newdbe is None or isinstance(newdbe,QtCore.QString):
			return
		myform = self.formFactory.getInstanceByDBEName(newdbe.getTypeName(),dbmgr=self.server)
		newdbe.readFKFrom( mydbe )
		newdbe.setDefaultValues(self.server)
		newdbe.setValue('father_id',mydbe.getValue('id'))
		newdbe.setValue('name',"New %s"%newdbe.getTypeName())
		icon = QtGui.QIcon()
		icon.addPixmap(QtGui.QPixmap(_fromUtf8(":/rPrj/%s"%myform.getDetailIcon())), QtGui.QIcon.Normal, QtGui.QIcon.Off)
		child = QtGui.QStandardItem( icon, "%s" % (newdbe.getValue('name') ) )
		child.setData( newdbe )
		myitem.appendRow( child )
		self.ui.treeView.setCurrentIndex( self.modelTree.indexFromItem(child) )
		self.emit(QtCore.SIGNAL("selected(QStandardItem,PyQt_PyObject)"),child,newdbe)
	def slotItemChanged(self,item):
		if DEBUG_WIDGET: print "slotItemChanged: item=%s" % (item)
		mydbe = item.data().toPyObject()
		myparent = item.parent()
		if myparent is None:
			return
		mydbeparent = myparent.data().toPyObject()
		if DEBUG_WIDGET: print "slotItemChanged: mydbe.father_id=%s" % (mydbe.getValue('father_id'))
		if DEBUG_WIDGET: print "slotItemChanged: mydbeparent.id=%s" % (mydbeparent.getValue('id'))
		if mydbe.getValue('father_id')!=mydbeparent.getValue('id'):
			if DEBUG_WIDGET: print "slotItemChanged: PARENT CHANGED"
			try:
				if self.server.canWrite(mydbe) and self.server.canExecute(mydbeparent):
					# Yes, we can... change father
					mydbe.readFKFrom( mydbeparent )
					mydbe.setValue('father_id',mydbeparent.getValue('id'))
					mydbe = self.server.update(mydbe)
				else:
					# Change not allowed
					myoldparent = self._findItemById( mydbe.getValue('father_id') )
					if not myoldparent is None:
						__old = myparent.takeRow( item.row() )
						myoldparent.appendRow(__old)
			except Exception,e:
				# Change not possible
				myoldparent = self._findItemById( mydbe.getValue('father_id') )
				if not myoldparent is None:
					__old = myparent.takeRow( item.row() )
					myoldparent.appendRow(__old)
				self.Alert("Unable to move item!",e)
				raise e
	def slotReload(self):
		msg = QtGui.QApplication.translate("TreeView", "Confirm Reload?", None, QtGui.QApplication.UnicodeUTF8)
		if not self.Confirm( msg ):
			return
		index = self.ui.treeView.currentIndex()
		myitem = self.modelTree.itemFromIndex(index)
		mydbe = myitem.data().toPyObject()
		if mydbe.isNew():
			return
		cerca = self.server.getClazzByTypeName('DBEObject')(attrs={'id':"%s" % mydbe.getValue('id')})
		lista = self.server.search(cerca)
		if len(lista)!=1:
			self.Alert("Error: fetched %d results."%(len(lista)))
			return
		mydbe = lista[0]
		self.runSearchChilds( myitem, True )
		self.emit(QtCore.SIGNAL("selected(QStandardItem,PyQt_PyObject)"),myitem,mydbe)
	def slotCustomAction(self):
		sender = self.sender()
		customActionString = "%s" % sender.data().toPyObject()
		customAction = "action%s%s" % (customActionString[0].upper(),customActionString[1:])
		index = self.ui.treeView.currentIndex()
		myitem = self.modelTree.itemFromIndex(index)
		mydbe = myitem.data().toPyObject()
		if customAction=='actionDownload':
			self.emit(QtCore.SIGNAL("actionDownload(PyQt_PyObject)"),mydbe)
			return
		try:
			getattr(self,customAction)(mydbe)
		except Exception,e:
			self.Alert("Error while executing '%s'." % sender.text(),e)
	def slotDelete(self):
		msg = QtGui.QApplication.translate("TreeView", "Confirm Delete?", None, QtGui.QApplication.UnicodeUTF8)
		if not self.Confirm( msg ):
			return
		index = self.ui.treeView.currentIndex()
		myitem = self.modelTree.itemFromIndex(index)
		myrow = myitem.index().row()
		myparent = myitem.parent()
		if myitem is None:
			return
		mydbe = myitem.data().toPyObject()
		mydbe = self.runDelete(mydbe)
		if not mydbe is None and not mydbe.isDeleted():
			self.SetStatusText("Delete: errors.")
			return
		myparent.removeRow(myrow)
		self.ui.treeView.setCurrentIndex( self.modelTree.indexFromItem(myparent) )
		self.previousItem = myparent
		mydbe = myparent.data().toPyObject()
		self.SetStatusText("Delete: OK")
		self.emit(QtCore.SIGNAL("selected(QStandardItem,PyQt_PyObject)"),myparent,mydbe)



# -*- coding: utf-8 -*-

#
# @copyright &copy; 2012 by Roberto Rocco Angeloni <roberto@roccoangeloni.it>
# @license http://opensource.org/licenses/lgpl-3.0.html GNU Lesser General Public License, version 3.0 (LGPLv3)
# @version $Id: editwidgetcontroller.py $
# @package apps
# 
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

"""
"""

import os
from PyQt4 import QtCore, QtGui, Qt
try:
	_fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
	_fromUtf8 = lambda s: s

class qtComponentException(Exception):
	pass

class qtComponent(QtCore.QObject):
	def __init__(self, config, parent=None):
		QtCore.QObject.__init__(self)
		self.config = config
		self.parent = parent
		
		self.logfile = file( self.getPref( self.__class__.__name__ ,"logfile", "%s%s.%s.log"%(os.path.expanduser("~"), os.path.sep,self.__class__.__name__) ) ,'a') #'w')
		
		self._statusText=""
		
		self.init()
		self.mythreads={}
		# QT
		QtCore.QObject.connect(self, QtCore.SIGNAL('setProgress(int)'), self.SetProgress )
		QtCore.QObject.connect(self, QtCore.SIGNAL('setStatusText(PyQt_PyObject)'), self.SetStatusText )
	def setParent(self,p):
		self.parent = p
# ############################## DESCRIPTION ###################################
	def getName(self):
		return self.__class__.__name__
	def getDescription(self):
		return self.__class__.__doc__
	def getIcon(self):
		raise qtComponentException("qtComponent.getIcon: NOT YET IMPLEMENTED!")
# ################################## GUI #######################################
	def getMenu(self,parent):
		return None
	def getPanel(self,parentWidget=None):
		raise qtComponentException("getPanel: NOT YET IMPLEMENTED!")
	def getPreferencesPanel(self):
		raise qtComponentException("getPreferencesPanel: NOT YET IMPLEMENTED!")
# ################################ COMPONENT ###################################
	def init(self):
		"""Sequenza di inizializzazione"""
		pass
	def finalize(self):
		"""Sequenza di terminazione"""
		pass
# ############################## PREFERENCES ###################################
	def SaveUI2Config(self):
		pass
	def RestoreUIFromConfig(self):
		pass
	def getPref(self,section,option,default=None):
		if not self.parent is None:
			return self.parent.getPref(section,option,default)
		if isinstance(default,bool): default="%s" % (default)
		if not self.config.has_section(section):
			self.config.add_section(section)
		if not self.config.has_option(section,option):# and default:
			self.config.set(section,option,default)
		return self.config.get(section,option,default)
	def setPref(self,section,option,valore):
		if not self.parent is None:
			self.parent.setPref(section,option,valore)
			return
		if not self.config.has_section(section):
			self.config.add_section(section)
		if isinstance( valore, bool ): valore ="%s"%(valore)
		self.config.set(section,option,valore)
	def SaveConfig(self):
		self.config.write( file(path_to_config,'w') )
# ############################## ... ###################################
	def Alert(self,msg, eccezione=None):
		if self.parent is None:
			return
		self.parent.Alert(msg,eccezione)
	def Confirm(self,msg):
		if self.parent is None:
			return True
		return self.parent.Confirm(msg)
	def SetStatusText(self,msg):
		if self.parent is None:
			return
		self.parent.SetStatusText(msg)
	def SetProgress(self,perc):
		if self.parent is None:
			return
		self.parent.SetProgress(perc)
# ################################ UTILITY ######################################
	def Log(self, message, level=0):
		self.logfile.write( "%s::%s> %s\n" % (datetime.datetime.now().strftime("%Y%m%d%H%M%S"), self.__class__.__name__, message) )
		self.logfile.flush()
	def SetStatusText(self,msg):
		self.parent.SetStatusText(msg)
		pass
	def emitStatusText(self,msg):
		self.emit(QtCore.SIGNAL('setStatusText(PyQt_PyObject)'),msg)
	def SetProgress(self,perc):
		self.parent.SetProgress(perc)
	def emitProgress(self,perc):
		self.emit(QtCore.SIGNAL('setProgress(int)'),perc)
# ############################# BUSINESS LOGIC #################################
	pass


class qtComponentContainer(qtComponent):
	def __init__(self, config, parent=None):
		qtComponent.__init__(self, config, parent)
		# BL
		self.components={}
	def addComponent(self,c):
		tmp_page = c.getPanel()
		self.components[c.getName()]=[c,tmp_page]
	pass



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'logindialog.ui'
#
# Created: Sun Apr  1 20:05:12 2012
#      by: PyQt4 UI code generator 4.9
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

class Ui_LoginDialog(object):
    def setupUi(self, LoginDialog):
        LoginDialog.setObjectName(_fromUtf8("LoginDialog"))
        LoginDialog.resize(436, 180)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(LoginDialog.sizePolicy().hasHeightForWidth())
        LoginDialog.setSizePolicy(sizePolicy)
        LoginDialog.setModal(True)
        self.gridLayout = QtGui.QGridLayout(LoginDialog)
        self.gridLayout.setSizeConstraint(QtGui.QLayout.SetMinimumSize)
        self.gridLayout.setMargin(8)
        self.gridLayout.setObjectName(_fromUtf8("gridLayout"))
        self.label = QtGui.QLabel(LoginDialog)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label.sizePolicy().hasHeightForWidth())
        self.label.setSizePolicy(sizePolicy)
        self.label.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTop|QtCore.Qt.AlignTrailing)
        self.label.setObjectName(_fromUtf8("label"))
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.lineEdit = QtGui.QLineEdit(LoginDialog)
        self.lineEdit.setObjectName(_fromUtf8("lineEdit"))
        self.gridLayout.addWidget(self.lineEdit, 0, 2, 1, 1)
        self.label_2 = QtGui.QLabel(LoginDialog)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_2.sizePolicy().hasHeightForWidth())
        self.label_2.setSizePolicy(sizePolicy)
        self.label_2.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTop|QtCore.Qt.AlignTrailing)
        self.label_2.setObjectName(_fromUtf8("label_2"))
        self.gridLayout.addWidget(self.label_2, 1, 0, 1, 1)
        self.lineEdit_2 = QtGui.QLineEdit(LoginDialog)
        self.lineEdit_2.setEchoMode(QtGui.QLineEdit.Password)
        self.lineEdit_2.setObjectName(_fromUtf8("lineEdit_2"))
        self.gridLayout.addWidget(self.lineEdit_2, 1, 2, 1, 1)
        self.label_3 = QtGui.QLabel(LoginDialog)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_3.sizePolicy().hasHeightForWidth())
        self.label_3.setSizePolicy(sizePolicy)
        self.label_3.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTop|QtCore.Qt.AlignTrailing)
        self.label_3.setObjectName(_fromUtf8("label_3"))
        self.gridLayout.addWidget(self.label_3, 4, 0, 1, 1)
        self.comboBox = QtGui.QComboBox(LoginDialog)
        self.comboBox.setEditable(True)
        self.comboBox.setSizeAdjustPolicy(QtGui.QComboBox.AdjustToMinimumContentsLength)
        self.comboBox.setMinimumContentsLength(10)
        self.comboBox.setObjectName(_fromUtf8("comboBox"))
        self.gridLayout.addWidget(self.comboBox, 4, 2, 1, 1)
        self.label_4 = QtGui.QLabel(LoginDialog)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_4.sizePolicy().hasHeightForWidth())
        self.label_4.setSizePolicy(sizePolicy)
        self.label_4.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTop|QtCore.Qt.AlignTrailing)
        self.label_4.setObjectName(_fromUtf8("label_4"))
        self.gridLayout.addWidget(self.label_4, 3, 0, 1, 1)
        self.lineEdit_schema = QtGui.QLineEdit(LoginDialog)
        self.lineEdit_schema.setObjectName(_fromUtf8("lineEdit_schema"))
        self.gridLayout.addWidget(self.lineEdit_schema, 3, 2, 1, 1)
        self.pushButton = QtGui.QPushButton(LoginDialog)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.pushButton.sizePolicy().hasHeightForWidth())
        self.pushButton.setSizePolicy(sizePolicy)
        self.pushButton.setCheckable(True)
        self.pushButton.setChecked(False)
        self.pushButton.setObjectName(_fromUtf8("pushButton"))
        self.gridLayout.addWidget(self.pushButton, 1, 3, 1, 1)
        self.buttonBox = QtGui.QDialogButtonBox(LoginDialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtGui.QDialogButtonBox.Cancel|QtGui.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName(_fromUtf8("buttonBox"))
        self.gridLayout.addWidget(self.buttonBox, 5, 0, 1, 4)

        self.retranslateUi(LoginDialog)
        QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL(_fromUtf8("accepted()")), LoginDialog.accept)
        QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL(_fromUtf8("rejected()")), LoginDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(LoginDialog)

    def retranslateUi(self, LoginDialog):
        LoginDialog.setWindowTitle(QtGui.QApplication.translate("LoginDialog", "Login", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("LoginDialog", "Login", None, QtGui.QApplication.UnicodeUTF8))
        self.label_2.setText(QtGui.QApplication.translate("LoginDialog", "Password", None, QtGui.QApplication.UnicodeUTF8))
        self.label_3.setText(QtGui.QApplication.translate("LoginDialog", "Server", None, QtGui.QApplication.UnicodeUTF8))
        self.label_4.setText(QtGui.QApplication.translate("LoginDialog", "Schema", None, QtGui.QApplication.UnicodeUTF8))
        self.pushButton.setToolTip(QtGui.QApplication.translate("LoginDialog", "More options", None, QtGui.QApplication.UnicodeUTF8))
        self.pushButton.setText(QtGui.QApplication.translate("LoginDialog", "+ / -", None, QtGui.QApplication.UnicodeUTF8))




# -*- coding: utf-8 -*-
#
# @copyright &copy; 2012 by Roberto Rocco Angeloni <roberto@roccoangeloni.it>
# @license http://opensource.org/licenses/lgpl-3.0.html GNU Lesser General Public License, version 3.0 (LGPLv3)
# @version $Id: xmlrpc.py $
# @package dblayer
# 
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#


from rprj.dblayer import DBConnectionProvider,DBLayerException,isDateTime
from rprj.net.jsonlib import JSONClient,Binary
import codecs,datetime,os,sys,traceback
from base64 import b64decode
#import threading # TEST

# 1 mega = 1048576
UPLOAD_SPLIT_SIZE = 1048576


class JsonConnectionProvider(DBConnectionProvider):
	def __init__(self, host, db, user, pwd, verbose=0):
		"""host: here is the url of the server
		db: TODO is meaningless here?"""
		DBConnectionProvider.__init__(self, host, db, user, pwd, verbose)
		self.verbose = verbose
		if self._host.startswith('jsons'):
			raise DBLayerException("TODO: Json via https")
		self._server = None
		self._cache = {}
		self.dbmgr = None # Needed by _jsonToDbe
		#self.semaforo = threading.Semaphore()
	def customInit(self):
		"""???"""
		pass
	def getConnection(self):
		#self.semaforo.acquire(True)
		if self._server is None:
			self._server = JSONClient(self._host)
		#self.semaforo.release()
		return self
	def freeConnection(self, conn):
		#self.semaforo.acquire(True)
		if not self._server is None:
			del self._server
		self._server = None
		#self.semaforo.release()
	def getDBType(self):
		return "Json"
	def isProxy(self):
		return True

	def _dbeToJson(self,dbe):
		ret={}
		for n in dbe.getNames():
			tmpvalue = dbe.getValue(n)
			if isinstance( tmpvalue, datetime.datetime ) or isinstance( tmpvalue, datetime.date ) or isinstance( tmpvalue, datetime.time ): # or isinstance(tmpvalue,Binary):
				ret[n] = "%s" % tmpvalue
			else:
				ret[n] = tmpvalue
		return [ dbe.getTypeName(), ret ]
	def _jsonToDbe(self, xmldbe):
		typename = xmldbe['_typename']
		xmldbe.pop('_typename')
		nomi = [ n for n in xmldbe.keys() ]
		valori = [ n for n in xmldbe.values() ]
		valori2=[]
		for v in valori:
			#if isinstance(v, Binary): v = v.data
			if isDateTime(v): #rra.dblayer.isDateTime(v):
				#"0000-00-00 00:00:00"
				v2=None
				try:
					v2 = datetime.datetime(int(v[:4]),int(v[5:7]),int(v[8:10]),int(v[11:13]),int(v[14:16]),int(v[17:19]))
				except Exception,e:
					try:
						v2 = datetime.datetime(int(v[:4]),int(v[5:7]),int(v[8:10]),int(v[11:13]),int(v[14:16]))
					except Exception, e:
						try:
							v2 = datetime.time(int(v[11:13]),int(v[14:16]),int(v[17:19]))
						except Exception,e:
							v2 = datetime.time(int(v[11:13]),int(v[14:16]))
				valori2.append( v2 )
			elif (type(v)==str or type(v)==unicode) and v.startswith("base64:"):
				valori2.append( b64decode( v[len("base64:"):] ) )
			else:
				valori2.append(v)
		#if isinstance(typename, Binary): typename = typename.data
		myclazz = self.dbmgr.getClazzByTypeName(typename)
		return myclazz( myclazz().getTableName(), names=nomi, values=valori2 )
	def getRegisteredTypes(self):
		"""Returns the registered types"""
		ret = {}
		try:
			#self.semaforo.acquire(True)
			msg, lista = self._server.getTypeList()
			#self.semaforo.release()
			self.lastMessages = msg.data
			for k,v in lista[0].iteritems():
				ret[k] = RemoteDBE(k,v.data,self)
			self._cache = ret
		except Exception, e:
			print "JsonConnectionProvider.getRegisteredTypes: ECCEZIONE ############# Inizio."
			print "ECCEZIONE: %s" % ( e )
			print "".join( traceback.format_tb(sys.exc_info()[2]) )
			print "JsonConnectionProvider.getRegisteredTypes: ECCEZIONE ############# Fine."
			raise DBLayerException( "Problemi di comunicazione col server: %s" % e )
		return ret
	def getClazzByTypeName(self, typename, case_sensitive=False):
		try:
			if self.verbose: print "JsonConnectionProvider.getClazzByTypeName: typename=%s" % ( typename )
			if self._cache is None: self._cache = self.getRegisteredTypes()
			ret = DBEntity
			if self._cache.has_key('default'):
				ret = self._cache['default']
			for k in self._cache.keys():
				if self.verbose: print "DBEFactory.getClazzByTypeName: k=%s" % ( k )
				mytypename = ''
				if isinstance(self._cache[k],RemoteDBE):
					mytypename=self._cache[k].getTypeName()
				else:
					mytypename=self._cache[k]().getTypeName()
				if case_sensitive:
					if mytypename==typename:
						ret = self._cache[k]
						break
				else:
					if mytypename.lower()==typename.lower():
						ret = self._cache[k]
						break
			return ret
		except Exception, e:
			print "JsonConnectionProvider.getClazzByTypeName: ECCEZIONE ############# Inizio."
			print "ECCEZIONE: %s" % ( e )
			print "".join( traceback.format_tb(sys.exc_info()[2]) )
			print "%s"%e
			print "JsonConnectionProvider.getClazzByTypeName: ECCEZIONE ############# Fine."
			return self._cache['default']
	def getClazz(self, tablename):
		ret = None
		if self._cache.has_key( tablename ):
			ret = self._cache[tablename]
		else:
			ret=self._cache['default']
		return ret
	def insert(self, dbe):
		# Before Insert
		# 2012.04.04 dbe._before_insert(self.dbmgr)
		# 2012.04.04 dbe = self.dbmgr._before_insert(dbe)
		# Insert
		dbexml = self._dbeToJson(dbe)
		#self.semaforo.acquire(True)
		tmp = self._server.insert( dbexml )
		print "tmp;", tmp
		#self.semaforo.release()
		tmparray = tmp[1]
		self.lastMessages = b64decode(tmp[0])
		dbe = self._jsonToDbe( tmparray[0] )
		# After Insert
		# 2012.04.04 dbe = self.dbmgr._after_insert(dbe)
		# 2012.04.04 dbe._after_insert(self.dbmgr)
		return dbe
	def update(self,dbe):
		# Before Update
		# 2012.04.04 dbe._before_update(self.dbmgr)
		# 2012.04.04 dbe = self.dbmgr._before_update(dbe)
		# Update
		dbexml = self._dbeToJson(dbe)
		#self.semaforo.acquire(True)
		tmp = self._server.update( dbexml )
		#self.semaforo.release()
		tmparray = tmp[1]
		self.lastMessages = b64decode(tmp[0])
		if len(tmparray)>0:
			dbe = self._jsonToDbe( tmparray[0] )
			# After Update
			# 2012.04.04 dbe = self.dbmgr._after_update(dbe)
			# 2012.04.04 dbe._after_update(self.dbmgr)
		else:
			dbe = None
		return dbe
	def select(self,tablename,searchString):
		if self.verbose: print "JsonConnectionProvider.select: tablename=%s" % (tablename)
		if self.verbose: print "JsonConnectionProvider.select: searchString=%s" % (searchString)
		if self.verbose: print "JsonConnectionProvider.select: start search=%s" % (datetime.datetime.now())
		try:
			#self.semaforo.acquire(True)
			tmp = self._server.select( tablename, searchString )
			#self.semaforo.release()
			res = tmp[1]
			self.lastMessages = b64decode(tmp[0])
		except DBLayerException, e:
			raise DBLayerException( "Problemi di comunicazione col server: %s" % e )
		if self.verbose: print "JsonConnectionProvider.select: start xml->dbe=%s" % (datetime.datetime.now())
		ret = []
		for r in res:
			tmpdbe = self._jsonToDbe(r)
			ret.append( tmpdbe )
		if self.verbose: print "JsonConnectionProvider.select: end time = %s" % (datetime.datetime.now())
		return ret
	def delete(self,dbe):
		# Delete
		dbexml = self._dbeToJson(dbe)
		#self.semaforo.acquire(True)
		tmp = self._server.delete( dbexml )
		#self.semaforo.release()
		tmparray = tmp[1]
		self.lastMessages = b64decode(tmp[0])
		if self.verbose: print "JsonConnectionProvider.delete: self.lastMessages = %s" % (self.lastMessages)
		dbe = self._jsonToDbe( tmparray[0] )
		return dbe
	def search(self, dbe, uselike=1, orderby=None, ignore_deleted=True, full_object=True):
		casesensitive = 0
		if self.verbose: print "JsonConnectionProvider.search: start time = %s" % (datetime.datetime.now())
		tablename = dbe.getTableName()
		# Search
		dbexml = self._dbeToJson(dbe)
		if self.verbose: print "JsonConnectionProvider.search: dbexml = %s" % (dbexml)
		if self.verbose: print "JsonConnectionProvider.search: uselike = %s" % (uselike)
		#if self.verbose: print "JsonConnectionProvider.search: casesensitive = %s" % (casesensitive)
		if self.verbose: print "JsonConnectionProvider.search: orderby = %s" % (orderby)
		try:
			if orderby is None: orderby=''
			#self.semaforo.acquire(True)
			msg, lista = self._server.search( dbexml, uselike, casesensitive, orderby, ignore_deleted, full_object )
			msg = b64decode(msg)
			#self.semaforo.release()
			if self.verbose: print "JsonConnectionProvider.search: msg=\n  ", "\n  ".join( ("%s"%msg).split("\n") )
			self.lastMessages = msg
		except Exception, e:
			print "JsonConnectionProvider.search: ECCEZIONE ############# Inizio."
			print "ECCEZIONE: %s" % ( e )
			print "".join( traceback.format_tb(sys.exc_info()[2]) )
			print "JsonConnectionProvider.search: ECCEZIONE ############# Fine."
			raise DBLayerException( "Problemi di comunicazione col server: %s" % e )
		ret = []
		for r in lista:
			tmpdbe = self._jsonToDbe(r)
			ret.append( tmpdbe )
		if self.verbose: print "JsonConnectionProvider.search: ret = %s" % ( len(ret) )
		if self.verbose: print "JsonConnectionProvider.search: end time = %s" % (datetime.datetime.now())
		return ret
	def copy(self, dbe):
		raise DBLayerException("copy: not yet implemented")

# ################### Remote Server ###################
	def ping(self):
		#self.semaforo.acquire(True)
		self.msg, ret = self._server.ping()
		#self.semaforo.release()
		if self.verbose:
			print "msg:",self.msg
			print "ret:",ret
		return ret[0]
	def login(self,user,pwd):
		#self.semaforo.acquire(True)
		self.msg, ret = self._server.login(user,pwd)
		#self.semaforo.release()
		if self.verbose:
			print "JsonConnectionProvider.login: msg=%s"%self.msg
			print "JsonConnectionProvider.login: ret=%s"%ret
		if len(ret)==1:
			tmpdbe = self._jsonToDbe(ret[0])
			# The remote server clean the password value in the returned dbe :-) more safe
			tmpdbe.setValue('pwd',pwd)
			return tmpdbe
		return None
	def getLoggedUser(self):
		#self.semaforo.acquire(True)
		self.msg, ret = self._server.getLoggedUser()
		self.msg = b64decode(self.msg)
		#self.semaforo.release()
		if self.verbose:
			print "JsonConnectionProvider.getLoggedUser: msg=%s"%self.msg
			print "JsonConnectionProvider.getLoggedUser: ret=%s"%ret
		if len(ret)==1:
			tmp = self._jsonToDbe(ret[0])
			if self.verbose: print "JsonConnectionProvider.getLoggedUser: tmp=%s"%tmp
			return tmp
		return None
	def getFormSchema(self,language='python',aClassname=''):
		#self.semaforo.acquire(True)
		myformschema, tmp = self._server.getFormSchema(language,aClassname)
		#self.semaforo.release()
		if self.verbose:
			print "myformschema:",myformschema
			print "tmp:",tmp
		return myformschema
	def getDBSchema(self,language='python',aClassname=''):
		#self.semaforo.acquire(True)
		myformschema, tmp = self._server.getDBSchema(language,aClassname)
		#self.semaforo.release()
		if self.verbose:
			print "myformschema:",myformschema
			print "tmp:",tmp
		return myformschema
	def objectById(self,myid,ignore_deleted=True):
		#self.semaforo.acquire(True)
		self.msg, ret = self._server.objectById(myid,ignore_deleted)
		self.msg = b64decode(self.msg)
		#self.semaforo.release()
		if self.verbose:
			print "msg:",self.msg
			print "ret:",ret
		if len(ret)==1:
			return self._jsonToDbe(ret[0])
		return None
	def fullObjectById(self,myid,ignore_deleted=True):
		#self.semaforo.acquire(True)
		ret=[]
		try:
			self.msg, ret = self._server.fullObjectById(myid,ignore_deleted)
			self.msg = b64decode(self.msg)
		except Exception,e:
			print "JsonConnectionProvider.fullObjectById: ECCEZIONE ############# Start."
			print "ECCEZIONE: %s" % ( e )
			print "".join( traceback.format_tb(sys.exc_info()[2]) )
			print "JsonConnectionProvider.fullObjectById: ECCEZIONE ############# End."
		finally:
			#self.semaforo.release()
			pass
		if self.verbose:
			print "msg:",self.msg
			print "ret:",ret
		if len(ret)==1:
			return self._jsonToDbe(ret[0])
		return None
	def objectByName(self,myid,ignore_deleted=True):
		#self.semaforo.acquire(True)
		ret=[]
		tmp = self._server.objectByName(myid,ignore_deleted)
		if len(tmp)>0:
			self.msg = b64decode(tmp[0])
		if len(tmp)>1:
			ret=tmp[1:][0]
		#self.semaforo.release()
		if self.verbose:
			print "msg:",self.msg
			print "ret:",ret
		# 2012.05.07: start.
		return [self._jsonToDbe(x) for x in ret]
		#if len(ret)==1:
		#	return self._jsonToDbe(ret[0])
		#return None
		# 2012.05.07: end.
	def fullObjectByName(self,myid,ignore_deleted=True):
		#self.semaforo.acquire(True)
		ret=[]
		try:
			tmp = self._server.objectByName(myid,ignore_deleted)
			if len(tmp)>0:
				self.msg = b64decode(tmp[0])
			if len(tmp)>1:
				ret=tmp[1:][0]
			#self.msg, ret = self._server.fullObjectByName(myid,ignore_deleted)
			#self.msg = b64decode(self.msg)
		except Exception,e:
			print "JsonConnectionProvider.fullObjectByName: ECCEZIONE ############# Inizio."
			print "ECCEZIONE: %s" % ( e )
			print "".join( traceback.format_tb(sys.exc_info()[2]) )
			print "JsonConnectionProvider.fullObjectByName: ECCEZIONE ############# Fine."
			raise DBLayerException( "Communication troubles with the remote server: %s" % e )
		finally:
			#self.semaforo.release()
			pass
		if self.verbose:
			print "msg:",self.msg
			print "ret:",ret
		# 2012.05.07: start.
		return [self._jsonToDbe(x) for x in ret]
		#if len(ret)==1:
		#	return self._jsonToDbe(ret[0])
		#return None
		# 2012.05.07: end.
	def uploadFile(self,local_filename):
		# Remote destination directory
		#self.semaforo.acquire(True)
		messaggio, ret = self._server.GetRandomUploadDestinationDirectory()
		messaggio = b64decode(messaggio)
		#self.semaforo.release()
		if self.verbose:
			print "JsonConnectionProvider.uploadFile: messaggio = %s" % ( messaggio )
			print "JsonConnectionProvider.uploadFile: ret = %s" % ( ret )
		dest_dir = ret['dest_dir']
		if self.verbose:
			print "JsonConnectionProvider.uploadFile: dest_dir:",dest_dir
		# Short local filename
		filename_corto = local_filename[ len(os.path.dirname(local_filename)): ]
		if filename_corto.startswith('/'):
			filename_corto = filename_corto[1:]
		if self.verbose: print "JsonConnectionProvider.uploadFile: filename_corto: ", filename_corto
		# Load file
		myfile = file( local_filename ).read()
		# Upload TODO split the file in chunks
		#self.semaforo.acquire(True)
		messaggio, ret = self._server.Upload(1,1,filename_corto,u"%s" % Binary(myfile), dest_dir)
		messaggio = b64decode(messaggio)
		#self.semaforo.release()
		if self.verbose:
			print "JsonConnectionProvider.uploadFile: messaggio = %s" % ( messaggio )
			print "JsonConnectionProvider.uploadFile: ret = %s" % ( ret )
		filename = ret['filename']
		dest_dir = ret['dest_dir']
		if self.verbose:
			print "JsonConnectionProvider.uploadFile: filename: ", filename
			print "JsonConnectionProvider.uploadFile: dest_dir: ", dest_dir
		return "%s/%s" % (dest_dir,filename)
		#raise DBLayerException("JsonConnectionProvider.uploadFile: not implemented.")
	def downloadFile(self,remote_filename,local_path,view_thumbnail=False):
		uuid = remote_filename
		#self.semaforo.acquire(True)
		messaggio, ret = self._server.Download(uuid,view_thumbnail)
		messaggio = b64decode(messaggio)
		#self.semaforo.release()
		if self.verbose:
			print "JsonConnectionProvider.downloadFile: messaggio = %s" % ( messaggio )
			tmp = {}
			for k in ret.keys():
				if k=='contents': continue
				tmp[k]=ret[k]
			print "JsonConnectionProvider.downloadFile: ret (senza contents) = %s" % ( tmp )
		mime = "%s" % ret['mime']
		filesize = ret['filesize']
		filename = "%s%s%s" % (local_path,os.path.sep,ret['filename'])
		if self.verbose:
			print "JsonConnectionProvider.downloadFile: mime = %s" % ( mime )
			print "JsonConnectionProvider.downloadFile: filesize = %s" % ( filesize )
			print "JsonConnectionProvider.downloadFile: filename = %s" % ( filename )
		file(filename,'wb').write( b64decode(ret['contents']) )
		return filename
